/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Index - IndexSettingsService.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.business;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import ch.unige.solidify.controller.IndexController;
import ch.unige.solidify.index.indexing.IndexingService;
import ch.unige.solidify.index.settings.IndexingSettingsService;
import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.model.index.IndexSettings;
import ch.unige.solidify.service.CompositeNoSqlResourceService;

@Service("IndexSettingsCompositeService")
@ConditionalOnBean(IndexController.class)
public class IndexSettingsService extends CompositeNoSqlResourceService<IndexSettings> {

  private IndexingSettingsService<IndexSettings> indexingSettingsService;
  private IndexingService<IndexMetadata> indexingService;

  public IndexSettingsService(
          IndexingSettingsService<IndexSettings> indexingSettingsService,
          IndexingService<IndexMetadata> indexingService) {
    this.indexingSettingsService = indexingSettingsService;
    this.indexingService = indexingService;
  }

  @Override
  public IndexSettings findOne(String id) {
    return this.indexingSettingsService.findOne(id);
  }

  @Override
  public List<IndexSettings> findAll(IndexSettings search) {
    return this.indexingSettingsService.findAll(search);
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    final IndexSettings indexSettings = this.findOne(parentId);
    if (childClass.equals(IndexMetadata.class)) {
      final Page<IndexMetadata> indexMetadataList = this.indexingService.findAll(indexSettings.getResId(), null, pageable);
      return (Page<V>) indexMetadataList;

    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String id) {
    final IndexSettings indexSettings = this.findOne(parentId);
    return (V) this.indexingService.findOne(indexSettings.getResId(), id);
  }

  @Override
  public IndexSettings save(IndexSettings t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public IndexSettings update(IndexSettings t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean delete(IndexSettings t) {
    throw new UnsupportedOperationException();
  }

}
