/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Model - Image.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.model;

import java.util.Arrays;
import java.util.Objects;

import javax.persistence.Lob;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.rest.ResourceNormalized;

@MappedSuperclass
public abstract class Image extends ResourceNormalized {

  private static final int DB_FILENAME_LENGTH = 2024;

  @JsonIgnore
  @Lob
  @NotNull
  private byte[] imageContent;

  @Size(max = DB_FILENAME_LENGTH)
  private String fileName;

  private Long fileSize;

  private String mimeType;

  protected Image() {
  }

  public void setImageContent(byte[] imageContent) {
    this.imageContent = imageContent;
  }

  public void setFileSize(Long fileSize) {
    this.fileSize = fileSize;
  }

  public void setFileName(String fileName) {
    this.fileName = fileName;
  }

  public byte[] getImageContent() {
    return this.imageContent;
  }

  public Long getFileSize() {
    return this.fileSize;
  }

  public String getFileName() {
    return this.fileName;
  }

  public String getMimeType() {
    return this.mimeType;
  }

  public void setMimeType(String mimeType) {
    this.mimeType = mimeType;
  }

  @Override
  public void init() {
    // Do nothing
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = super.hashCode();
    result = prime * result + Arrays.hashCode(this.imageContent);
    result = prime * result + Objects.hash(this.fileName, this.fileSize);
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (!super.equals(obj))
      return false;
    if (getClass() != obj.getClass())
      return false;
    Image other = (Image) obj;
    return Objects.equals(this.fileName, other.fileName)
            && Objects.equals(this.fileSize, other.fileSize)
            && Arrays.equals(this.imageContent, other.imageContent);
  }

}
