/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Util - ZipTest.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.solidify.test.util;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.util.FileTool;
import ch.unige.solidify.util.ZipTool;

class ZipTest {
  private final static String DEST_PATH1 = "dlcm.xml";

  private final static String DEST_PATH2 = "include/datacite-funderIdentifierType-v4.xsd";

  private final static String FILE_NAME_3 = "geo-location.json";
  private final static String INCLUDE = "include";
  private static final Logger log = LoggerFactory.getLogger(ZipTest.class);
  private final static String PATH_FILE1 = "dlcm/dlcm.xml";
  private final static String PATH_FILE2 = "dlcm/include/datacite-funderIdentifierType-v4.xsd";
  private final static String PATH_FILE3 = "json/geo-location.json";
  private final static String UNZIP_DIR = "unzip";
  private final static String ZIP_ARCHIVE = "test.zip";
  private final static String ZIP_DIR = "dlcm";
  private final static int ZIP_FILE_NUMBER = 17;
  private final static int ZIP_INCLUDE_NUMBER = 8;
  private final static int ZIP_XSD_NUMBER = 9;

  @Test
  void createZipByListFilesTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(".").getURI());
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<Path> filesToAdd = new ArrayList<>();
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE1).getURI()));
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE2).getURI()));
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE3).getURI()));
    assertTrue(zipTool.addFilesToZip(zipDir, filesToAdd, null, false));
    assertTrue(zipTool.checkZip());
    assertEquals(3, zipTool.getEntryNumber());
    assertTrue(zipTool.contains(PATH_FILE3));
    assertFalse(zipTool.contains(FILE_NAME_3));
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
  }

  @Test
  void createZipSpecifyingListFilesByDifferentPathsTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(".").getURI());
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<Path> filesToAdd = new ArrayList<>();
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE1).getURI()));
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE2).getURI()));
    final List<Path> destPaths = new ArrayList<>();
    destPaths.add(Paths.get(DEST_PATH1));
    destPaths.add(Paths.get(DEST_PATH2));
    assertTrue(zipTool.addFilesToZip(zipDir, filesToAdd, destPaths, true));
    assertTrue(zipTool.checkZip());
    assertEquals(2, zipTool.getEntryNumber());
    assertTrue(zipTool.contains(DEST_PATH1));
    assertFalse(zipTool.contains(PATH_FILE1));
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
  }

  @Test
  void createZipSpecifyingListFilesTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(".").getURI());
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<Path> filesToAdd = new ArrayList<>();
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE1).getURI()));
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE2).getURI()));
    filesToAdd.add(Paths.get(new ClassPathResource(PATH_FILE3).getURI()));
    assertTrue(zipTool.addFilesToZip(zipDir, filesToAdd, null, true));
    assertTrue(zipTool.checkZip());
    assertEquals(3, zipTool.getEntryNumber());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
  }

  @Test
  void partialUnzipArchiveTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<String> extractList = Arrays.asList("dlcm.xsd",
            "include" + SolidifyConstants.URL_SEP + "datacite-");
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.unzipFiles(unzipDir, extractList));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    fileList.addAll(FileTool.readFolder(unzipDir.resolve(INCLUDE)));
    assertEquals(ZIP_XSD_NUMBER, fileList.size());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void partialUnzipArchiveTest2() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<String> extractList = Arrays.asList("dlcm.xsd",
            "include" + SolidifyConstants.URL_SEP + "datacite-");
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.unzipFiles(unzipDir, extractList));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    fileList.addAll(FileTool.readFolder(unzipDir.resolve(INCLUDE)));
    assertEquals(ZIP_XSD_NUMBER, fileList.size());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void partialUnzipArchiveTest2bis() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    final List<String> extractList = Arrays.asList("dlcm.xsd",
            "include" + SolidifyConstants.URL_SEP + "datacite-");
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.unzipFiles(unzipDir, extractList));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    fileList.addAll(FileTool.readFolder(unzipDir.resolve(INCLUDE)));
    assertEquals(ZIP_XSD_NUMBER, fileList.size());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void unzipArchiveTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertTrue(zipTool.zipFiles(zipDir.toUri()));
    assertTrue(zipTool.unzipFiles(unzipDir));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    fileList.addAll(FileTool.readFolder(unzipDir.resolve(INCLUDE)));
    assertEquals(ZIP_FILE_NUMBER, fileList.size());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void unzipArchiveTest1() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.unzipFiles(unzipDir));
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void unzipArchiveTest2() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.unzipFiles(unzipDir, null));
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir));
  }

  @Test
  void unzipFolderTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertTrue(zipTool.zipFiles(zipDir.toUri()));
    assertTrue(zipTool.unzipFolder(unzipDir, INCLUDE + SolidifyConstants.URL_SEP));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    assertEquals(ZIP_INCLUDE_NUMBER, fileList.size(), "Wrong file number");
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir), "Cannot delete folder");
  }

  @Test
  void unzipOSDependantFolderTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final Path unzipDir = Paths
            .get(System.getProperty("java.io.tmpdir") + File.separatorChar + UNZIP_DIR);
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertTrue(zipTool.zipFiles(zipDir.toUri()));
    assertTrue(zipTool.unzipFolder(unzipDir, INCLUDE + File.separator));
    final List<Path> fileList = FileTool.readFolder(unzipDir);
    assertEquals(ZIP_INCLUDE_NUMBER, fileList.size(),"Wrong file number");
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
    assertTrue(FileTool.deleteFolder(unzipDir),"Wrong file number");
  }

  @Test
  void zipArchiveTest() throws IOException {
    final Path zipDir = Paths.get(new ClassPathResource(ZIP_DIR).getURI());
    final ZipTool zipTool = new ZipTool(ZIP_ARCHIVE);
    assertNotNull(zipDir);
    assertTrue(zipTool.zipFiles(zipDir));
    assertTrue(zipTool.checkZip());
    assertEquals(ZIP_FILE_NUMBER, zipTool.getEntryNumber());
    FileTool.deleteFile(Paths.get(ZIP_ARCHIVE));
  }

}
